﻿// AudibleDragControl by Charles Petzold, November 2009
using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Media;
using NAudio.Wave;

namespace AudibleDragging
{
    public partial class AudibleDragControl : UserControl
    {
        WaveOut waveOut;
        PortamentoSineWaveOscillator oscillator;

        public AudibleDragControl()
        {
            InitializeComponent();
        }

        void OnThumbDragStarted(object sender, DragStartedEventArgs args)
        {
            BringToTop();

            if ((waveOut = new WaveOut()) != null)
            {
                double pitch = CalculatePitch();
                oscillator = new PortamentoSineWaveOscillator(44100, pitch);
                oscillator.Amplitude = 8192;
                waveOut.Init(oscillator);
                waveOut.Play();
            }
        }

        void OnThumbDragDelta(object sender, DragDeltaEventArgs args)
        {
            Canvas.SetLeft(this, args.HorizontalChange + Canvas.GetLeft(this));
            Canvas.SetTop(this, args.VerticalChange + Canvas.GetTop(this));
            oscillator.Pitch = CalculatePitch();
        }

        void OnThumbDragCompleted(object sender, DragCompletedEventArgs args)
        {
            if (waveOut != null)
            {
                waveOut.Stop();
                waveOut.Dispose();
                waveOut = null;
            }
        }

        double CalculatePitch()
        {
            double x = Canvas.GetLeft(this);
            double y = Canvas.GetTop(this);
            double distance = Math.Sqrt(x * x + y * y);

            // Middle C plus one octave per 200 units
            return 60 + 12 * distance / 200;
        }

        void BringToTop()
        {
            DependencyObject parent = VisualTreeHelper.GetParent(this);
            int zLo = Int16.MaxValue, zHi = Int16.MinValue;

            if (parent != null && parent is Panel)
            {
                Panel panel = parent as Panel;

                foreach (UIElement child in panel.Children)
                {
                    if (child != this)
                    {
                        zLo = Math.Min(zLo, Panel.GetZIndex(child));
                        zHi = Math.Max(zHi, Panel.GetZIndex(child));
                    }
                }

                foreach (UIElement child in panel.Children)
                {
                    if (child == this)
                        Panel.SetZIndex(child, zHi - zLo + 1);
                    else
                        Panel.SetZIndex(child, Panel.GetZIndex(child) - zLo);
                }
            }
        }
    }
}
